@@ -14,8 +14,10 @@ from weakref import WeakValueDictionary
 
 import numpy as np
 
-from . import _path, cbook, rcParams
+import matplotlib as mpl
+from . import _path, cbook
 from .cbook import _to_unmasked_float_array, simple_linear_interpolation
+from .bezier import BezierSegment
 
 
 class Path:
@@ -119,24 +121,27 @@ class Path:
             intended for public use.
         closed : bool, optional
             If *codes* is None and closed is True, vertices will be treated as
-            line segments of a closed polygon.
+            line segments of a closed polygon.  Note that the last vertex will
+            then be ignored (as the corresponding code will be set to
+            CLOSEPOLY).
         readonly : bool, optional
             Makes the path behave in an immutable way and sets the vertices
             and codes as read-only arrays.
         """
         vertices = _to_unmasked_float_array(vertices)
-        if vertices.ndim != 2 or vertices.shape[1] != 2:
-            raise ValueError(
-                "'vertices' must be a 2D list or array with shape Nx2")
+        cbook._check_shape((None, 2), vertices=vertices)
 
         if codes is not None:
             codes = np.asarray(codes, self.code_type)
             if codes.ndim != 1 or len(codes) != len(vertices):
                 raise ValueError("'codes' must be a 1D list or array with the "
-                                 "same length of 'vertices'")
+                                 "same length of 'vertices'. "
+                                 f"Your vertices have shape {vertices.shape} "
+                                 f"but your codes have shape {codes.shape}")
             if len(codes) and codes[0] != self.MOVETO:
                 raise ValueError("The first element of 'code' must be equal "
-                                 "to 'MOVETO' ({})".format(self.MOVETO))
+                                 f"to 'MOVETO' ({self.MOVETO}).  "
+                                 f"Your first code is {codes[0]}")
         elif closed and len(vertices):
             codes = np.empty(len(vertices), dtype=self.code_type)
             codes[0] = self.MOVETO
@@ -181,15 +186,15 @@ class Path:
             pth._interpolation_steps = internals_from._interpolation_steps
         else:
             pth._should_simplify = True
-            pth._simplify_threshold = rcParams['path.simplify_threshold']
+            pth._simplify_threshold = mpl.rcParams['path.simplify_threshold']
             pth._interpolation_steps = 1
         return pth
 
     def _update_values(self):
-        self._simplify_threshold = rcParams['path.simplify_threshold']
+        self._simplify_threshold = mpl.rcParams['path.simplify_threshold']
         self._should_simplify = (
             self._simplify_threshold > 0 and
-            rcParams['path.simplify'] and
+            mpl.rcParams['path.simplify'] and
             len(self._vertices) >= 128 and
             (self._codes is None or np.all(self._codes <= Path.LINETO))
         )
@@ -239,15 +244,6 @@ class Path:
     def simplify_threshold(self, threshold):
         self._simplify_threshold = threshold
 
-    @cbook.deprecated(
-        "3.1", alternative="not np.isfinite(self.vertices).all()")
-    @property
-    def has_nonfinite(self):
-        """
-        `True` if the vertices array has nonfinite values.
-        """
-        return not np.isfinite(self._vertices).all()
-
     @property
     def should_simplify(self):
         """
@@ -268,7 +264,7 @@ class Path:
 
     def __copy__(self):
         """
-        Returns a shallow copy of the `Path`, which will share the
+        Return a shallow copy of the `Path`, which will share the
         vertices and codes with the source `Path`.
         """
         import copy
@@ -278,7 +274,7 @@ class Path:
 
     def __deepcopy__(self, memo=None):
         """
-        Returns a deepcopy of the `Path`.  The `Path` will not be
+        Return a deepcopy of the `Path`.  The `Path` will not be
         readonly, even if the source `Path` is.
         """
         try:
@@ -322,18 +318,15 @@ class Path:
 
     @classmethod
     def make_compound_path(cls, *args):
-        """Make a compound path from a list of Path objects."""
+        """
+        Make a compound path from a list of Path objects. Blindly removes all
+        Path.STOP control points.
+        """
         # Handle an empty list in args (i.e. no args).
         if not args:
             return Path(np.empty([0, 2], dtype=np.float32))
-
-        lengths = [len(x) for x in args]
-        total_length = sum(lengths)
-
-        vertices = np.vstack([x.vertices for x in args])
-        vertices.reshape((total_length, 2))
-
-        codes = np.empty(total_length, dtype=cls.code_type)
+        vertices = np.concatenate([x.vertices for x in args])
+        codes = np.empty(len(vertices), dtype=cls.code_type)
         i = 0
         for path in args:
             if path.codes is None:
@@ -342,6 +335,10 @@ class Path:
             else:
                 codes[i:i + len(path.codes)] = path.codes
             i += len(path.vertices)
+        # remove STOP's, since internal STOPs are a bug
+        not_stop_mask = codes != cls.STOP
+        vertices = vertices[not_stop_mask, :]
+        codes = codes[not_stop_mask]
 
         return cls(vertices, codes)
 
@@ -355,9 +352,10 @@ class Path:
                       snap=False, stroke_width=1.0, simplify=None,
                       curves=True, sketch=None):
         """
-        Iterates over all of the curve segments in the path.  Each iteration
-        returns a 2-tuple ``(vertices, code)``, where ``vertices`` is a
-        sequence of 1-3 coordinate pairs, and ``code`` is a `Path` code.
+        Iterate over all curve segments in the path.
+
+        Each iteration returns a pair ``(vertices, code)``, where ``vertices``
+        is a sequence of 1-3 coordinate pairs, and ``code`` is a `Path` code.
 
         Additionally, this method can provide a number of standard cleanups and
         conversions to the path.
@@ -375,7 +373,7 @@ class Path:
             defining a rectangle in which to clip the path.
         snap : None or bool, optional
             If True, snap all nodes to pixels; if False, don't snap them.
-            If None, perform snapping if the path contains only segments
+            If None, snap if the path contains only segments
             parallel to the x or y axes, and no more than 1024 of them.
         stroke_width : float, optional
             The width of the stroke being drawn (used for path snapping).
@@ -416,6 +414,54 @@ class Path:
                     curr_vertices = np.append(curr_vertices, next(vertices))
             yield curr_vertices, code
 
+    def iter_bezier(self, **kwargs):
+        """
+        Iterate over each bezier curve (lines included) in a Path.
+
+        Parameters
+        ----------
+        **kwargs
+            Forwarded to `.iter_segments`.
+
+        Yields
+        ------
+        B : matplotlib.bezier.BezierSegment
+            The bezier curves that make up the current path. Note in particular
+            that freestanding points are bezier curves of order 0, and lines
+            are bezier curves of order 1 (with two control points).
+        code : Path.code_type
+            The code describing what kind of curve is being returned.
+            Path.MOVETO, Path.LINETO, Path.CURVE3, Path.CURVE4 correspond to
+            bezier curves with 1, 2, 3, and 4 control points (respectively).
+            Path.CLOSEPOLY is a Path.LINETO with the control points correctly
+            chosen based on the start/end points of the current stroke.
+        """
+        first_vert = None
+        prev_vert = None
+        for verts, code in self.iter_segments(**kwargs):
+            if first_vert is None:
+                if code != Path.MOVETO:
+                    raise ValueError("Malformed path, must start with MOVETO.")
+            if code == Path.MOVETO:  # a point is like "CURVE1"
+                first_vert = verts
+                yield BezierSegment(np.array([first_vert])), code
+            elif code == Path.LINETO:  # "CURVE2"
+                yield BezierSegment(np.array([prev_vert, verts])), code
+            elif code == Path.CURVE3:
+                yield BezierSegment(np.array([prev_vert, verts[:2],
+                                              verts[2:]])), code
+            elif code == Path.CURVE4:
+                yield BezierSegment(np.array([prev_vert, verts[:2],
+                                              verts[2:4], verts[4:]])), code
+            elif code == Path.CLOSEPOLY:
+                yield BezierSegment(np.array([prev_vert, first_vert])), code
+            elif code == Path.STOP:
+                return
+            else:
+                raise ValueError("Invalid Path.code_type: " + str(code))
+            prev_vert = verts[-2:]
+
+    @cbook._delete_parameter("3.3", "quantize")
     def cleaned(self, transform=None, remove_nans=False, clip=None,
                 quantize=False, simplify=False, curves=False,
                 stroke_width=1.0, snap=False, sketch=None):
@@ -514,59 +560,68 @@ class Path:
 
     def contains_path(self, path, transform=None):
         """
-        Returns whether this (closed) path completely contains the given path.
+        Return whether this (closed) path completely contains the given path.
 
         If *transform* is not ``None``, the path will be transformed before
-        performing the test.
+        checking for containment.
         """
         if transform is not None:
             transform = transform.frozen()
         return _path.path_in_path(self, None, path, transform)
 
-    def get_extents(self, transform=None):
+    def get_extents(self, transform=None, **kwargs):
         """
-        Returns the extents (*xmin*, *ymin*, *xmax*, *ymax*) of the path.
+        Get Bbox of the path.
 
-        Unlike computing the extents on the *vertices* alone, this
-        algorithm will take into account the curves and deal with
-        control points appropriately.
+        Parameters
+        ----------
+        transform : matplotlib.transforms.Transform, optional
+            Transform to apply to path before computing extents, if any.
+        **kwargs
+            Forwarded to `.iter_bezier`.
+
+        Returns
+        -------
+        matplotlib.transforms.Bbox
+            The extents of the path Bbox([[xmin, ymin], [xmax, ymax]])
         """
         from .transforms import Bbox
-        path = self
         if transform is not None:
-            transform = transform.frozen()
-            if not transform.is_affine:
-                path = self.transformed(transform)
-                transform = None
-        return Bbox(_path.get_path_extents(path, transform))
+            self = transform.transform_path(self)
+        bbox = Bbox.null()
+        for curve, code in self.iter_bezier(**kwargs):
+            # places where the derivative is zero can be extrema
+            _, dzeros = curve.axis_aligned_extrema()
+            # as can the ends of the curve
+            bbox.update_from_data_xy(curve([0, *dzeros, 1]), ignore=False)
+        return bbox
 
     def intersects_path(self, other, filled=True):
         """
-        Returns *True* if this path intersects another given path.
+        Return whether if this path intersects another given path.
 
-        *filled*, when True, treats the paths as if they were filled.
-        That is, if one path completely encloses the other,
-        :meth:`intersects_path` will return True.
+        If *filled* is True, then this also returns True if one path completely
+        encloses the other (i.e., the paths are treated as filled).
         """
         return _path.path_intersects_path(self, other, filled)
 
     def intersects_bbox(self, bbox, filled=True):
         """
-        Returns whether this path intersects a given `~.transforms.Bbox`.
+        Return whether this path intersects a given `~.transforms.Bbox`.
 
-        *filled*, when True, treats the path as if it was filled.
-        That is, if the path completely encloses the bounding box,
-        :meth:`intersects_bbox` will return True.
+        If *filled* is True, then this also returns True if the path completely
+        encloses the `.Bbox` (i.e., the path is treated as filled).
 
         The bounding box is always considered filled.
         """
-        return _path.path_intersects_rectangle(self,
-            bbox.x0, bbox.y0, bbox.x1, bbox.y1, filled)
+        return _path.path_intersects_rectangle(
+            self, bbox.x0, bbox.y0, bbox.x1, bbox.y1, filled)
 
     def interpolated(self, steps):
         """
-        Returns a new path resampled to length N x steps.  Does not
-        currently handle interpolating curves.
+        Return a new path resampled to length N x steps.
+
+        Codes other than LINETO are not handled correctly.
         """
         if steps == 1:
             return self
@@ -633,12 +688,8 @@ class Path:
         Return a `Path` instance of the unit rectangle from (0, 0) to (1, 1).
         """
         if cls._unit_rectangle is None:
-            cls._unit_rectangle = \
-                cls([[0.0, 0.0], [1.0, 0.0], [1.0, 1.0], [0.0, 1.0],
-                     [0.0, 0.0]],
-                    [cls.MOVETO, cls.LINETO, cls.LINETO, cls.LINETO,
-                     cls.CLOSEPOLY],
-                    readonly=True)
+            cls._unit_rectangle = cls([[0, 0], [1, 0], [1, 1], [0, 1], [0, 0]],
+                                      closed=True, readonly=True)
         return cls._unit_rectangle
 
     _unit_regular_polygons = WeakValueDictionary()
@@ -647,7 +698,8 @@ class Path:
     def unit_regular_polygon(cls, numVertices):
         """
         Return a :class:`Path` instance for a unit regular polygon with the
-        given *numVertices* and radius of 1.0, centered at (0, 0).
+        given *numVertices* such that the circumscribing circle has radius 1.0,
+        centered at (0, 0).
         """
         if numVertices <= 16:
             path = cls._unit_regular_polygons.get(numVertices)
@@ -659,11 +711,7 @@ class Path:
                      # "points-up".
                      + np.pi / 2)
             verts = np.column_stack((np.cos(theta), np.sin(theta)))
-            codes = np.empty(numVertices + 1)
-            codes[0] = cls.MOVETO
-            codes[1:-1] = cls.LINETO
-            codes[-1] = cls.CLOSEPOLY
-            path = cls(verts, codes, readonly=True)
+            path = cls(verts, closed=True, readonly=True)
             if numVertices <= 16:
                 cls._unit_regular_polygons[numVertices] = path
         return path
@@ -688,12 +736,8 @@ class Path:
             theta += np.pi / 2.0
             r = np.ones(ns2 + 1)
             r[1::2] = innerCircle
-            verts = np.vstack((r*np.cos(theta), r*np.sin(theta))).transpose()
-            codes = np.empty((ns2 + 1,))
-            codes[0] = cls.MOVETO
-            codes[1:-1] = cls.LINETO
-            codes[-1] = cls.CLOSEPOLY
-            path = cls(verts, codes, readonly=True)
+            verts = (r * np.vstack((np.cos(theta), np.sin(theta)))).T
+            path = cls(verts, closed=True, readonly=True)
             if numVertices <= 16:
                 cls._unit_regular_stars[(numVertices, innerCircle)] = path
         return path
@@ -727,10 +771,10 @@ class Path:
 
         Parameters
         ----------
-        center : pair of floats
-            The center of the circle. Default ``(0, 0)``.
-        radius : float
-            The radius of the circle. Default is 1.
+        center : (float, float), default: (0, 0)
+            The center of the circle.
+        radius : float, default: 1
+            The radius of the circle.
         readonly : bool
             Whether the created path should have the "readonly" argument
             set when creating the Path instance.
@@ -740,7 +784,7 @@ class Path:
         The circle is approximated using 8 cubic Bezier curves, as described in
 
           Lancaster, Don.  `Approximating a Circle or an Ellipse Using Four
-          Bezier Cubic Splines <http://www.tinaja.com/glib/ellipse4.pdf>`_.
+          Bezier Cubic Splines <https://www.tinaja.com/glib/ellipse4.pdf>`_.
         """
         MAGIC = 0.2652031
         SQRTHALF = np.sqrt(0.5)
@@ -969,7 +1013,7 @@ def get_path_collection_extents(
     master_transform : `~.Transform`
         Global transformation applied to all paths.
     paths : list of `Path`
-    transform : list of `~.Affine2D`
+    transforms : list of `~.Affine2D`
     offsets : (N, 2) array-like
     offset_transform : `~.Affine2D`
         Transform applied to the offsets before offsetting the path.
@@ -989,23 +1033,3 @@ def get_path_collection_extents(
     return Bbox.from_extents(*_path.get_path_collection_extents(
         master_transform, paths, np.atleast_3d(transforms),
         offsets, offset_transform))
-
-
-@cbook.deprecated("3.1", alternative="get_paths_collection_extents")
-def get_paths_extents(paths, transforms=[]):
-    """
-    Given a sequence of :class:`Path` objects and optional
-    :class:`~matplotlib.transforms.Transform` objects, returns the
-    bounding box that encapsulates all of them.
-
-    *paths* is a sequence of :class:`Path` instances.
-
-    *transforms* is an optional sequence of
-    :class:`~matplotlib.transforms.Affine2D` instances to apply to
-    each path.
-    """
-    from .transforms import Bbox, Affine2D
-    if len(paths) == 0:
-        raise ValueError("No paths provided")
-    return Bbox.from_extents(*_path.get_path_collection_extents(
-        Affine2D(), paths, transforms, [], Affine2D()))
